Package: voluModel
Title: Modeling Species Distributions in Three Dimensions
Version: 0.2.5
Authors@R: 
    c(person(given = "Hannah L.",
             family = "Owens",
             role = c("aut", "cre", "cph"),
             email = "hannah.owens@gmail.com",
             comment = c(ORCID = "0000-0003-0071-1745")),
      person(given = "Emmaline",
             family = "Sheahan",
             role = "aut",
             email = "sheahane@ufl.edu",
             comment = c(ORCID = "0000-0003-3358-9758")),
      person(given = "Carsten", 
             family = "Rahbek",,,
             role="aut",
             comment = c(ORCID = "0000-0003-4585-0300")))
Maintainer: Hannah L. Owens <hannah.owens@gmail.com>
Description: Facilitates modeling species' ecological niches and 
  geographic distributions based on occurrences and environments that 
  have a vertical as well as horizontal component, and projecting models 
  into three-dimensional geographic space. Working in three dimensions is 
  useful in an aquatic context when the organisms one wishes to model can 
  be found across a wide range of depths in the water column. The package
  also contains functions to automatically generate marine training
  model training regions using machine learning, and interpolate and smooth
  patchily sampled environmental rasters using thin plate splines.
  Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016) <doi:10.1038/ncomms11484>.
  Nychka D, Furrer R, Paige J, Sain S (2021) <doi:10.5065/D6W957CT>.
  Pateiro-Lopez B, Rodriguez-Casal A (2022) <https://CRAN.R-project.org/package=alphahull>.
License: GPL-3
URL: https://hannahlowens.github.io/voluModel/,
        https://github.com/hannahlowens/voluModel
BugReports: https://github.com/hannahlowens/voluModel/issues
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: dplyr, fields, ggplot2, ggtext, grDevices, methods, metR,
        predicts, rangeBuilder (>= 2.0), rnaturalearth, terra,
        viridisLite, sf
Suggests: testthat (>= 3.0.0), nlme, knitr, covr, gridExtra, lattice,
        rJava, rmarkdown, rnaturalearthdata, tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-23 15:52:43 UTC; HannahOwens
Author: Hannah L. Owens [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-0071-1745>),
  Emmaline Sheahan [aut] (ORCID: <https://orcid.org/0000-0003-3358-9758>),
  Carsten Rahbek [aut] (ORCID: <https://orcid.org/0000-0003-4585-0300>)
Repository: CRAN
Date/Publication: 2026-07-23 16:50:02 UTC
