% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{transpColor}
\alias{transpColor}
\title{Transparent Color}
\usage{
transpColor(color, percent = 50)
}
\arguments{
\item{color}{Anything that can be interpreted by \code{rgb}
as a color.}

\item{percent}{A whole number between 0 and 100 specifying
how transparent the color should be.}
}
\value{
A \code{character} string with hex color, including
adjustment for transparency.
}
\description{
Generates transparent colors
}
\examples{

transpColor(color = "red", percent = 50)

}
\keyword{internal}
\keyword{plotting}
